#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

FILE *fsrc;
FILE *ftab;

int offs_src = 0;
int quoted = 1;

static void copy(int do_copy)
{
	int c;
	c = fgetc(fsrc);
	if (c < 0) {
		fprintf(stderr, "Premature end of C source\n");
		exit(1);
	}
	offs_src++;
	if (do_copy)
		fputc(c, stdout);
}

#if 0
static void copy_sep()
{

	for(;;) {
		int c;
		c = fgetc(fsrc);
		if (c < 0)
			return;
		if (!isspace(c) && (c != '}') && (c != '{')) {
			ungetc(c, fsrc);
			return;
		}
		offs_src++;
		fputc(c, stdout);
	}
}
#endif

static void copy_up_to(int offs, int do_copy)
{
	while(offs_src < offs)
		copy(do_copy);
}

static void put_text(const char *text)
{
	if (quoted) {
		char *s = unescape_dup(text);
		printf("%s", s);
		free(s);
	}
	else
		printf("%s", text);
}

int main(int argc, char *argv[])
{
	char *fn_src = NULL, *cmd;
	int c, n;

	for(n = 1; n < argc; n++) {
		cmd = argv[n];
		if (*cmd == '-') {
			while(*cmd == '-') cmd++;
			switch(*cmd) {
				case 'r': /* raw */
					quoted = 0;
					break;
				default:
					fprintf(stderr, "Unrecognized argument: %s\n", cmd);
					exit(1);
			}
		}
		else {
			if (fn_src != NULL) {
				fprintf(stderr, "Multiple source files specified (%s, %s)\n", fn_src, cmd);
				exit(1);
			}
			fn_src = cmd;
		}
	}

	if (fn_src == NULL) {
		fprintf(stderr, "No source file specified\n");
		exit(1);
	}

	fsrc = fopen(fn_src, "rb");
	ftab = stdin;

	while(!(feof(ftab))) {
		int from, to;
		char line[65536], *cmd, *arg, *text, *end;

		*line = '\0';
		fgets(line, sizeof(line), stdin);
		cmd = line;
		arg = strpbrk(cmd, " \t\r\n");
		if (arg != NULL) {
			*arg = '\0';
			arg++;
			from = strtol(arg, &end, 0);
		}
		else
			continue;
		if (!isspace(*end)) {
			fprintf(stderr, "Syntax error in edit tab: expected integer in the second column\n");
			exit(1);
		}
		arg = end+1;

		/* copy to where the next command starts */
		copy_up_to(from, 1);

		text = arg;
		end = text + strlen(text) - 1;
		while((*end == '\n') || (*end == '\r')) { *end = '\0'; end--; }


		if (strcmp(cmd, "insert") == 0) {
			put_text(text);
		}
		else {
			to = strtol(arg, &end, 0);
			text = end + 1;
			if (!isspace(*end)) {
				fprintf(stderr, "Syntax error in edit tab: expected integer in the third column\n");
				exit(1);
			}

			if (strcmp(cmd, "delete") == 0)
				copy_up_to(to, 0);
			else if (strcmp(cmd, "replace") == 0) {
				copy_up_to(to, 0);
				put_text(text);
			}
			else {
				fprintf(stderr, "Syntax error in edit tab: invalid command %s\n", cmd);
				exit(1);
			}
		}
	}

	/* copy the rest, if any */
	while((c = fgetc(fsrc)) != EOF) {
		offs_src++;
		fputc(c, stdout);
	}

	return 0;
}
